local addonName, addon = ...
local E, L, V, P, G, _ = unpack(ElvUI)
local PPBA = E:NewModule(addonName)

local barTexture = E.media.normTex
if barTexture == "Interface\\Addons\\SharedMedia\\statusbar\\Empty" then
	barTexture = "Interface\\AddOns\\ElvUI\\media\\textures\\normTex2.tga"
end
local staticTextures = {
	[0] = {nil,1,1,1},
	[1] = {nil,1,1,1}, --replaced below by CreateBackdrop in UnitPowerBarAlt_ApplyTextures
	[2] = {barTexture,1,1,1},
	[3] = {nil,1,1,1},
	[4] = {barTexture,1,1,1},
}


local function UnitAlternatePowerTextureInfo(...)
	local unit, textureIndex, timerIndex = ...
	local texture, r, g, b = unpack(staticTextures[textureIndex])
	local barColor = { r = 1, g = 1, b = 1 }
	if E.db.general.altpower.color == "UNITFRAME" then
		barColor = E.db.unitframe.colors.health
	elseif E.db.general.altpower.color == "POWER" then
		barColor = E.db.unitframe.colors.power[select(2,UnitPowerType(unit))]
	elseif E.db.general.altpower.color == "CLASS" then
		barColor = RAID_CLASS_COLORS[unit=="player" and E.myclass or select(2,UnitClass(unit))]
	elseif E.db.general.altpower.color == "CUSTOM" then
		barColor = E.db.general.altpower.customcolor
	end
	return texture, barColor.r, barColor.g, barColor.b
end

local function UnitAlternatePowerInfo(...)
	local unit = ...
	local barType, minPower, startInset, endInset, smooth, hideFromOthers, showOnRaid, opaqueSpark, opaqueFlash, powerName, powerTooltip, costString, barID = _G.UnitAlternatePowerInfo(...)
	if E.db.general.altpower.smooth == "ALWAYS" then
		smooth = true
	elseif E.db.general.altpower.smooth == "NEVER" then
		smooth = false
	end
	return (barType and ALT_POWER_TYPE_HORIZONTAL), minPower, 0, 0, smooth, hideFromOthers, showOnRaid, opaqueSpark, opaqueFlash, powerName, powerTooltip, costString, barID
end
local function GetAlternatePowerInfoByID(...)
	local barID = ...
	local barType, minPower, startInset, endInset, smooth, hideFromOthers, showOnRaid, opaqueSpark, opaqueFlash, powerName, powerTooltip, costString = _G.GetAlternatePowerInfoByID(...)
	if E.db.general.altpower.smooth == "ALWAYS" then
		smooth = true
	elseif E.db.general.altpower.smooth == "NEVER" then
		smooth = false
	end
	return (barType and ALT_POWER_TYPE_HORIZONTAL), minPower, 0, 0, smooth, hideFromOthers, showOnRaid, opaqueSpark, opaqueFlash, powerName, powerTooltip, costString
end



local UnitPowerBarAlt_ApplyTextures, UnitPowerBarAlt_SetUp, UnitPowerBarAlt_UpdateAll, UnitPowerBarAlt_Horizontal_SetUp
local PlayerBuffTimerManager_GetTimer, PlayerBuffTimerManager_UpdateTimers
--[[the below is copied and/or altered from FrameXML]]--

local altPowerBarTextures = {
    frame = 0,
    background = 1,
    fill = 2,
    spark = 3,
    flash = 4,
}

local ALT_POWER_BAR_PLAYER_SIZES = {
    [ALT_POWER_TYPE_HORIZONTAL]     = {x = 128, y = 50}, --from ElvUI\modules\blizzard\altpower.lua
}
 
function ElvUI_UnitPowerBarAlt_OnEvent(self, event, ...)
    local arg1, arg2 = ...;
     
    if ( event == self.updateAllEvent ) then
        UnitPowerBarAlt_UpdateAll(self);
    elseif ( event == "UNIT_POWER_BAR_SHOW" ) then
        if ( arg1 == self.unit ) then
            UnitPowerBarAlt_UpdateAll(self);
        end
    elseif ( event == "UNIT_POWER_BAR_HIDE" ) then
        if ( arg1 == self.unit ) then
            UnitPowerBarAlt_UpdateAll(self);
        end
    elseif ( event == "UNIT_POWER" ) then
        if ( arg1 == self.unit and arg2 == "ALTERNATE" ) then
            local currentPower = UnitPower(self.unit, ALTERNATE_POWER_INDEX);
            UnitPowerBarAlt_SetPower(self, currentPower);
        end
    elseif ( event == "UNIT_MAXPOWER" ) then
        if ( arg1 == self.unit and arg2 == "ALTERNATE" ) then
            local barType, minPower = UnitAlternatePowerInfo(self.unit);
            UnitPowerBarAlt_SetMinMaxPower(self, minPower, UnitPowerMax(self.unit, ALTERNATE_POWER_INDEX));
             
            local currentPower = UnitPower(self.unit, ALTERNATE_POWER_INDEX);
            UnitPowerBarAlt_SetPower(self, currentPower, true);
        end
    elseif ( event == "PLAYER_ENTERING_WORLD" ) then
        UnitPowerBarAlt_UpdateAll(self);
    end
end
 
UnitPowerBarAlt_ApplyTextures = function(frame, unit)
    for textureName, textureIndex in pairs(altPowerBarTextures) do
        local texture = frame[textureName];
        local texturePath, r, g, b = UnitAlternatePowerTextureInfo(unit, textureIndex, frame.timerIndex);
        texture:SetTexture(texturePath);
        texture:SetVertexColor(r, g, b, E.db.general.altpower.customalpha);
    end
	if not frame.backdrop then
		frame:CreateBackdrop(E.db.general.altpower.backdropcolor)
	else
		frame.backdrop:SetTemplate(E.db.general.altpower.backdropcolor)
	end
end
 
UnitPowerBarAlt_SetUp = function(self, barID)
    local barType, minPower, startInset, endInset, smooth, hideFromOthers, showOnRaid, opaqueSpark, opaqueFlash, powerName, powerTooltip, costString;
    if ( barID ) then
        barType, minPower, startInset, endInset, smooth, hideFromOthers, showOnRaid, opaqueSpark, opaqueFlash, powerName, powerTooltip, costString = GetAlternatePowerInfoByID(barID);
    else
        barType, minPower, startInset, endInset, smooth, hideFromOthers, showOnRaid, opaqueSpark, opaqueFlash, powerName, powerTooltip, costString, barID = UnitAlternatePowerInfo(self.unit);
    end
     
    self.startInset = startInset;
    self.endInset = endInset;
    self.smooth = smooth;
    self.powerName = powerName;
    self.powerTooltip = powerTooltip;
     
    local sizeInfo = ALT_POWER_BAR_PLAYER_SIZES[barType];
	
    self:SetSize(sizeInfo.x * self.scale, sizeInfo.y * self.scale);
     
    UnitPowerBarAlt_HideTextures(self); --It's up to the SetUp functions to show textures they need.
 
    self.statusFrame.enabled = true;
    UnitPowerBarAltStatus_ToggleFrame(self.statusFrame);
 
    if ( barType == ALT_POWER_TYPE_HORIZONTAL ) then
        UnitPowerBarAlt_Horizontal_SetUp(self);
    else
        error("Currently unhandled bar type: "..(barType or "nil"));
    end
     
    if ( opaqueSpark ) then
        self.spark:SetBlendMode("BLEND");
    else
        self.spark:SetBlendMode("ADD");
    end
     
    if ( opaqueFlash ) then
        self.flash:SetBlendMode("BLEND");
    else
        self.flash:SetBlendMode("ADD");
    end
     
    self:RegisterUnitEvent("UNIT_POWER", self.unit);
    self:RegisterUnitEvent("UNIT_MAXPOWER", self.unit);
end
 
UnitPowerBarAlt_UpdateAll = function(self)
    local barType, minPower, startInset, endInset, smooth, hideFromOthers, showOnRaid = UnitAlternatePowerInfo(self.unit);
    if ( barType and (not hideFromOthers or self.isPlayerBar) ) then
        UnitPowerBarAlt_TearDown(self);
        UnitPowerBarAlt_SetUp(self);
 
        local currentPower = UnitPower(self.unit, ALTERNATE_POWER_INDEX);
		local maxPower = UnitPowerMax(self.unit, ALTERNATE_POWER_INDEX);
		UnitPowerBarAlt_SetMinMaxPower(self, minPower, maxPower);
		 
		UnitPowerBarAlt_SetPower(self, currentPower, true);
        self:Show();
    else
        UnitPowerBarAlt_TearDown(self);
        self:Hide();
    end
end
 
--Horizontal Status Bar
UnitPowerBarAlt_Horizontal_SetUp = function(self)
    UnitPowerBarAlt_ApplyTextures(self, self.unit);
     
    self.frame:Show();
    self.background:Show();
    self.fill:Show();
    self.spark:Show();
     
    self.spark:ClearAllPoints();
    self.spark:SetHeight(self:GetHeight());
    self.spark:SetWidth(self:GetHeight()/8);
    self.spark:SetPoint("LEFT", self.fill, "RIGHT", -3 * self.scale, 0);
     
    self.fill:ClearAllPoints();
    self.fill:SetPoint("TOPLEFT");
    self.fill:SetPoint("BOTTOMLEFT");
    self.fill:SetWidth(self:GetWidth());
     
    self.UpdateFill = UnitPowerBarAlt_Horizontal_UpdateFill;
end
 
 
 
---------------------------------
-------- Buff Timer Code --------
---------------------------------
local PlayerBuffTimers = {}
local numBuffTimers = 0;
 
 
function ElvUI_PlayerBuffTimerManager_OnEvent(self, event, ...)
    local arg1 = ...;
    if ( arg1 == self.unit and event == "UNIT_POWER_BAR_TIMER_UPDATE" ) then
        PlayerBuffTimerManager_UpdateTimers(self);
    elseif ( event == "PLAYER_ENTERING_WORLD" ) then
        PlayerBuffTimerManager_UpdateTimers(self);
    end
end
 
 
PlayerBuffTimerManager_GetTimer = function(barType)
    local timerFrame;
     
    for i=1,numBuffTimers do
        local frame = _G["BuffTimer"..i];
        if ( frame and not frame:IsShown() ) then
            timerFrame = frame;
            break;
        end
    end
     
    if ( not timerFrame ) then
        numBuffTimers = numBuffTimers + 1;
		timerFrame = CreateFrame("Frame", "BuffTimer"..numBuffTimers, UIParent, "ElvUI_UnitPowerBarAltTemplate");
		timerFrame.scale = 1.0;
        timerFrame.unit = "player"
        timerFrame:SetScript("OnEvent", nil);
    end
     
    timerFrame:SetScript("OnUpdate", PlayerBuffTimer_OnUpdate);
    return timerFrame;
end
 
 
PlayerBuffTimerManager_UpdateTimers = function(self)
    for _, timer in pairs(PlayerBuffTimers) do
        timer.flagForHide = true;
    end
 
    local index = 1;
    local anchorFrame = PlayerPowerBarAlt;
    local duration, expiration, barID, auraID = UnitPowerBarTimerInfo("player", index);
    while ( barID ) do
        if ( not PlayerBuffTimers[auraID] ) then -- this timer is new. add it
            local barType = GetAlternatePowerInfoByID(barID);
            local timer = PlayerBuffTimerManager_GetTimer(barType);
            timer.timerIndex = index;
			UnitPowerBarAlt_TearDown(timer);
			UnitPowerBarAlt_SetUp(timer, barID);
			UnitPowerBarAlt_SetMinMaxPower(timer, 0, duration);
			UnitPowerBarAlt_SetPower(timer, duration, true);
             
            timer.timerExpiration = expiration;
            timer:Show();
            PlayerBuffTimers[auraID] = timer;
        end
 
         
        PlayerBuffTimers[auraID]:SetPoint("BOTTOM", anchorFrame, "TOP", 0, 32);
        anchorFrame = PlayerBuffTimers[auraID];
        PlayerBuffTimers[auraID].flagForHide = false;
        PlayerBuffTimers[auraID].timerIndex = index;
        index = index + 1;
        duration, expiration, barID, auraID = UnitPowerBarTimerInfo("player", index);
    end
     
    for auraID, timer in pairs(PlayerBuffTimers) do
        if ( timer.flagForHide ) then
            PlayerBuffTimers[auraID] = nil;
			UnitPowerBarAlt_TearDown(timer);
            timer:Hide();
        end
    end
end

--[[the above is copied and/or altered from FrameXML]]--

function PPBA:EnableDisable()
	if self.db.enable then
		if not self.bar:IsEventRegistered("PLAYER_ENTERING_WORLD") then
			UnitPowerBarAlt_Initialize(self.bar, "player", 1)
			UnitPowerBarAlt_UpdateAll(self.bar)
		end
	else
		self.bar:UnregisterAllEvents()
		UnitPowerBarAlt_TearDown(self.bar)
		self.bar:Hide()
	end
end

function PPBA:UpdateDimensions()
	local w, h = self.db.width, self.db.height
	ALT_POWER_BAR_PLAYER_SIZES[ALT_POWER_TYPE_HORIZONTAL] = {x = w, y = h}
	UnitPowerBarAlt_UpdateAll(self.bar)
	self:EnableDisable()
	self.holder:Size(w, h)
end

function PPBA:UpdateBarColor()
	UnitPowerBarAlt_ApplyTextures(self.bar, "player")
end