-- Generate my Ace Addon
local GlobalCooldownBar = LibStub("AceAddon-3.0"):NewAddon("GlobalCooldownBar", "AceEvent-3.0")

-- Defaults for GlobalCooldownBarDB
local defaults =
{
	global =
	{
		x = 750,
		y = 750,
		w = 400,
		h = 3,
		color = { r=0, g=1.0, b=0, a=1.0 },
		backgroundColor = { r=0.3, g=0.3, b=0.3, a=0.5 },
		lagColor = { r=1.0, g=0, b=0, a=1.0 },
		spark = true,
		combatOnly = true,
	}
}

-- Easy DB reference
local globalDB

-- The frame itself and the main texture
local gcdBarFrame

-- Variables for the spell being monitored
local start, duration

-- Options table for use of Ace-Config 3
local options =
{
    name = "Global Cooldown Bar",
    type = "group",
    args =
	{
		firstheader =
		{
			order = 1,
			name = "Position & Size",
			type = "header",
		},
		xposition =
		{
			order = 2,
			name = "Horizontal Position",
			desc = "Horizontal position of global cooldown bar on screen.",
			type = "input",
			set = 
			function(info, value)
				globalDB.x = value
				GlobalCooldownBar:SetupFrame()
			end,
			get = function(info) return globalDB.x end,
		},
		yposition =
		{
			order = 3,
			name = "Vertical Position",
			desc = "Vertical position of global cooldown bar on screen.",
			type = "input",
			set =
			function(info, value)
				globalDB.y = value
				GlobalCooldownBar:SetupFrame()
			end,
			get = function(info) return globalDB.y end,
		},
		width =
		{
			order = 4,
			name = "Width",
			desc = "Width of global cooldown bar.",
			type = "range",
			min = 50,
			max = 1000,
			step = 1,
			set =
			function(info, value)
				globalDB.w = value
				GlobalCooldownBar:SetupFrame()
			end,
			get = function(info) return globalDB.w end,
		},
		height =
		{
			order = 5,
			name = "Height",
			desc = "Height of global cooldown bar.",
			type = "range",
			min = 2,
			max = 30,
			step = 1,
			set = 
			function(info, value)
				globalDB.h = value
				GlobalCooldownBar:SetupFrame()
			end,
			get = function(info) return globalDB.h end,
		},
		secondheader =
		{
			order = 6,
			name = "Colors",
			type = "header",
		},
		color =
		{
			order = 7,
			name = "Color",
			desc = "Color of bar.",
			type = "color",
			hasAlpha = true,
			set = function(info, r, g, b ,a)
				globalDB.color.r = r
				globalDB.color.g = g
				globalDB.color.b = b
				globalDB.color.a = a
				GlobalCooldownBar:SetupFrame()
			end,
			get = function()
				return globalDB.color.r, globalDB.color.g, globalDB.color.b, globalDB.color.a
			end,
		},
		backgroundColor =
		{
			order = 8,
			name = "Background Color",
			desc = "Background color of bar.",
			type = "color",
			hasAlpha = true,
			set = function(info, r, g, b, a)
				globalDB.backgroundColor.r = r
				globalDB.backgroundColor.g = g
				globalDB.backgroundColor.b = b
				globalDB.backgroundColor.a = a
				GlobalCooldownBar:SetupFrame()
			end,
			get = function()
				return globalDB.backgroundColor.r, globalDB.backgroundColor.g, globalDB.backgroundColor.b, globalDB.backgroundColor.a
			end,
		},
		lagColor =
		{
			order = 9,
			name = "Lag Color",
			desc = "Color of lag part of bar.",
			type = "color",
			hasAlpha = true,
			set = function(info, r, g, b, a)
				globalDB.lagColor.r = r
				globalDB.lagColor.g = g
				globalDB.lagColor.b = b
				globalDB.lagColor.a = a
				GlobalCooldownBar:SetupFrame()
			end,
			get = function()
				return globalDB.lagColor.r, globalDB.lagColor.g, globalDB.lagColor.b, globalDB.lagColor.a
			end,
		},
		thirdheader =
		{
			order = 10,
			name = "Other",
			type = "header",
		},
		spark =
		{
			order = 11,
			name = "Spark",
			desc = "Toggle to use a spark on cooldown bar or not.",
			type = "toggle",
			set = function(info, value) globalDB.spark = value end,
			get = function(info) return globalDB.spark end,
		},
		combatOnly =
		{
			order = 12,
			name = "Combat Only",
			desc = "Toggle to use a the global cooldown bar in combat only.",
			type = "toggle",
			set = function(info, value) globalDB.combatOnly = value end,
			get = function(info) return globalDB.combatOnly end,
		},
    },
}

function GlobalCooldownBar:OnInitialize()
	-- Register the DataBase
	self.db = LibStub("AceDB-3.0"):New("GlobalCooldownBarDB", defaults, true);
	globalDB = self.db.global
	
	-- And register the options
	LibStub("AceConfig-3.0"):RegisterOptionsTable("Global Cooldown Bar", options, "gcdbar");

	-- And add the options table to the actual interface UI
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Global Cooldown Bar", "Global Cooldown Bar")

	-- Add the event
	GlobalCooldownBar:RegisterEvent("ACTIONBAR_UPDATE_COOLDOWN")
	
	-- Get meta data to output version number
	print("Global Cooldown Bar v" .. GetAddOnMetadata(GlobalCooldownBar:GetName(), "Version") .. " loaded");
end

function GlobalCooldownBar:SetupFrame()
	if gcdBarFrame then
		gcdBarFrame:Hide()
		gcdBarFrame = nil
	end
	
	gcdBarFrame = CreateFrame("Frame", nil, UIParent)

	gcdBarFrame:SetFrameStrata("BACKGROUND")
	gcdBarFrame:SetSize(globalDB.w, globalDB.h)
	gcdBarFrame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", globalDB.x, globalDB.y * -1)
	
	gcdBarFrame:SetScript('OnUpdate', OnUpdate)
	
	-- Back drop - this is the 'gray bar' under the gcd bar
	gcdBarFrame.backdropTexture = gcdBarFrame:CreateTexture(nil, "BACKGROUND")
	gcdBarFrame.backdropTexture:SetTexture(globalDB.backgroundColor.r, globalDB.backgroundColor.g, globalDB.backgroundColor.b, globalDB.backgroundColor.a)
	gcdBarFrame.backdropTexture:SetAllPoints(gcdBarFrame)
	
	-- Lag frame
	gcdBarFrame.lagBarTexture = gcdBarFrame:CreateTexture(nil, "BACKGROUND")
	gcdBarFrame.lagBarTexture:SetTexture(globalDB.lagColor.r, globalDB.lagColor.g, globalDB.lagColor.b, globalDB.lagColor.a)
	gcdBarFrame.lagBarTexture:SetHeight(gcdBarFrame:GetHeight())
	
	-- Create main bar itself
	gcdBarFrame.barTexture = gcdBarFrame:CreateTexture(nil, "BACKGROUND")
	gcdBarFrame.barTexture:SetTexture(globalDB.color.r, globalDB.color.g, globalDB.color.b, globalDB.color.a)
	gcdBarFrame.barTexture:SetHeight(gcdBarFrame:GetHeight())
	gcdBarFrame.barTexture:SetPoint("LEFT", gcdBarFrame, "LEFT")
	
	-- Create spark overlay
	gcdBarFrame.sparkTexture = gcdBarFrame:CreateTexture(nil, "OVERLAY")
	gcdBarFrame.sparkTexture:SetTexture("Interface\\CastingBar\\UI-CastingBar-Spark")
	gcdBarFrame.sparkTexture:SetBlendMode("ADD")
end

function GlobalCooldownBar:ACTIONBAR_UPDATE_COOLDOWN()
	-- Check the signature spell for the class
	-- local _, class = UnitClass("player")
	
	-- 61304 is the 'Global Cooldown' spell
	start, duration = GetSpellCooldown(61304)
	
	-- Check for combat status
	if (UnitAffectingCombat("player") == 1 or globalDB.combatOnly == false) and duration > 0 and duration <= 1.5 then
		-- Make the frame if it isn't already there
		if not gcdBarFrame then
			GlobalCooldownBar:SetupFrame()
		end

		-- Check for showing spark
		if globalDB.spark then
			gcdBarFrame.sparkTexture:Show()
		else
			gcdBarFrame.sparkTexture:Hide()
		end

		-- Get the lag
		local _, _, homeLag, worldLag = GetNetStats()
		local width = gcdBarFrame:GetSize()
		
		-- Now set the proportion of the lag of the duration and indicate it
		-- http://www.encyclopedia.com/topic/Reaction_Time.aspx
		-- 180 ms visual reaction time
		local lagBarLength
		
		if worldLag + 180 >= duration * 1000 then
			lagBarLength = width
		else
			lagBarLength = width * (worldLag + 180) / (duration * 1000)
		end
		
		gcdBarFrame.lagBarTexture:SetPoint("RIGHT", gcdBarFrame, "RIGHT")
		gcdBarFrame.lagBarTexture:SetWidth(lagBarLength)
		
		-- Show the frame, which will cause it's update to start getting events
		gcdBarFrame:Show()
	end
end

function OnUpdate(self, elapsed)
	if GetTime() - start < duration then
		-- Find the percentage complete
		local percentage = (GetTime() - start) / duration
		local width, height = gcdBarFrame:GetSize()

		-- Show the bar
		gcdBarFrame.barTexture:SetWidth(width * percentage)
		
		-- Set the spark at the proper position
		if globalDB.spark then
			gcdBarFrame.sparkTexture:SetPoint("CENTER", gcdBarFrame, "LEFT", width * percentage, 1)
		end
	else
		gcdBarFrame:Hide()
	end
end
